import java.awt.*;

class ZugPanel
  extends Panel
  implements ResetListener, DeblockListener
{
  private final static boolean DEBUG = false, RESET_ON_CRASH = false;

  Panel controls1, controls2;
  Panel lowerpart;

  ZugStrecke zugstrecke;

  Button quit=null, newtrain, faster, slower, stop, go, reset, info,
         version1, version2, version0;

  Label label;
  Resources rsrc;

  public void requestReset()
  {
    if(!reset.isEnabled()) reset.enable();
  }

  public void deblock()
  {
    if(!newtrain.isEnabled()) newtrain.enable();

    if(DEBUG) System.out.println("Deblockierung fuer neuen Zug aufgehoben");
  }

  ZugPanel()
  {
    rsrc=Resources.instance();
    setLayout(new BorderLayout(0,0));
    controls1 = new Panel();
    controls1.add(newtrain = new Button(rsrc.newtrain_str));
    controls1.add(faster = new Button(rsrc.faster_str));
    controls1.add(slower = new Button(rsrc.slower_str));
    controls1.add(stop = new Button(rsrc.stop_str));
    controls1.add(go = new Button(rsrc.go_str));

    controls2 = new Panel();
    controls2.add(info = new Button(rsrc.info_str));
    controls2.add(version0 = new Button("Version 1.0"));
    controls2.add(version1 = new Button("Version 1.1"));
    controls2.add(version2 = new Button("Version 1.2"));
    controls2.add(reset = new Button(rsrc.reset_str));

    if(RESET_ON_CRASH) reset.disable();

    if(!Resources.instance().startedAsApplet())
      controls2.add(quit = new Button(rsrc.quit_str));

    lowerpart = new Panel();
    lowerpart.setLayout(new BorderLayout(0,0));
    lowerpart.add("North", controls1);
    lowerpart.add("Center", controls2);

    if(Resources.instance().version == 0)
      {
        label = new Label(rsrc.backexplenation, Label.CENTER);
        lowerpart.add("South", label);
      }

    zugstrecke = new ZugStrecke(this,this);
    add("Center", zugstrecke);
    add("South", lowerpart);

    zugstrecke.newLok();
    newtrain.disable();
    zugstrecke.running = true;

    if(zugstrecke.running) { stop.enable(); go.disable(); }
    else { stop.disable(); go.enable(); }
  }
  
  private void newtrain()
  {
    zugstrecke.newLok();
    newtrain.disable();
  }

  private void reset()
  {
    zugstrecke.reset();
    if(RESET_ON_CRASH) reset.disable();
    repaint();
    newtrain();
  }
  
  public boolean handleEvent(Event e)
  {
    switch(e.id)
      {
	case Event.ACTION_EVENT:
	  if(e.target == info)
	    {
	      Frame infoframe = new Info("INFO", Resources.instance().info_data);
	      infoframe.resize(400,400);
	      infoframe.show();
	      return true;
	    }
	  else
	  if(e.target == version0)
	    {
	      Resources.instance().version = 0;
	      reset();
	      return true;
	    }
	  else
	  if(e.target == version1)
	    {
	      Resources.instance().version = 1;
	      reset();
	      return true;
	    }
	  else
	  if(e.target == version2)
	    {
	      Resources.instance().version = 2;
	      reset();
	      return true;
	    }
	  else
	  if(e.target == newtrain)
	    {
	      newtrain();
	      return true;
	    }
	  else
	  if(e.target == faster)
	    { 
	      zugstrecke.faster();
	      return true;
	    }
	  else
	  if(e.target == slower)
	    {
	      zugstrecke.slower();
	      return true;
	    }
	  else
	  if(e.target == stop)
	    {
              stop.disable(); go.enable();
	      zugstrecke.running = false;
	      return true;
	    }
	  else
	  if(e.target == go)
	    {
              stop.enable(); go.disable();
	      zugstrecke.running = true;
	      return true;
	    }
	  else
	  if(quit != null && e.target == quit)
	    {
	      Resources.instance().die();
	      return true;
	    }
	  else
	  if(e.target == reset)
	    {
	      reset();
	      return true;
	    }
	  break;
	  
	case Event.KEY_PRESS:
	  /*
	  if(DEBUG)
	    System.out.println("ZugPanel.handleEvent(KEY_PRESS)");
	  */

	  if(e.key == rsrc.quit_char) Resources.instance().die();
	  else
	  if(e.key == rsrc.faster_char) zugstrecke.faster();
	  else
	  if(e.key == rsrc.slower_char) zugstrecke.slower();
	  else
	  if(e.key == rsrc.newtrain_char)
	    {
	      if(newtrain.isEnabled())
	        {
	          zugstrecke.newLok();
		  newtrain.disable();
		}
	    }
	  else
	  if(e.key == rsrc.stop_char)
	    {
              stop.disable(); go.enable();
	      zugstrecke.running = false;
	    }
	  else
	  if(e.key == rsrc.go_char)
	    {
              stop.enable(); go.disable();
	      zugstrecke.running = true;
	    }
	  else
	  if(e.key == rsrc.reset_char)
	    {
              if(reset.isEnabled()) zugstrecke.reset();
	      if(RESET_ON_CRASH) reset.disable();
	    }

           return true;		// handle all key events in this component

        case Event.MOUSE_DOWN:
          zugstrecke.running = !zugstrecke.running;
          if(zugstrecke.running) { stop.enable(); go.disable(); }
          else { stop.disable(); go.enable(); }
	  return true;

        default:
	  break;
      }

    return false;
  }
}
